@@ -44,7 +44,6 @@ import getpass
 import shutil
 import stat
 
-
 import logging
 
 # logging.basicConfig(level=logging.DEBUG, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)
@@ -68,7 +67,7 @@ import logging
 #  Copy 'run.py' and 'settings.ini' files wherever you want to have your record repository.
 #  Windows let's you double click 'run.py'. Can run from desktop.
 # Magic number 42 is used to connect to all radios found in 'settings.ini'.
-print('0.6.62')
+print('0.6.68')
 
 # https://www.scrapehero.com/how-to-fake-and-rotate-user-agents-using-python-3/
 proxy_support = urllib.request.ProxyHandler({})
@@ -130,7 +129,7 @@ def signal_handler(sig, frame):
 class GBase:
     # class attribute
     exit_app = False
-    sleeper = 2   # for exit of all threads
+    sleeper = 2  # for exit of all threads
     pool = ThreadPoolExecutor(200)
     radio_base_dir = os.path.dirname(os.path.abspath(__file__)) + '//radiostations'  # if not set set_radio_base_dir()
     settings_path = os.path.dirname(os.path.abspath(__file__)) + '//settings.ini'  # if not set in set_settings_path()
@@ -223,6 +222,7 @@ class GIni(GBase):
                 print('\t>> %-20s <<' % _)
 
             print(' \n Radio stations in your list. ')
+            print(' Please use "Ctrl + C" to stop. ')
         return
 
     @staticmethod
@@ -529,42 +529,43 @@ class GRecorder:
         stream_song_name = GRecorder.current_song_dict[ini_key]
         GIni.fail_meta_dict[url] = 'False'
         headers = {'Connection': 'keep-alive'}
-        request = GNet.http_pool.request('GET', url, preload_content=False, retries=10, timeout=3000, headers=headers)
+        # request = GNet.http_pool.request('GET', url, preload_content=False, retries=10, timeout=3000, headers=headers)
 
         while not GBase.exit_app:
-            hd = request.headers  # try keep conn. open
-
-            rand = random.randrange(5, 16, 1)   # sleep random servers (5, 29, 1), helped not much in snap container
-            if GIni.srv_param_dict[ini_key + '_is_unknown_server'] == 'True':
-                if not GIni.fail_meta_dict[url] == 'True':  # let record go in one file
-                    pass
-                    ret = GRecorder.get_metadata_from_stream(url, ini_key)
-                    stream_song_name = ret
-
-            if GIni.srv_param_dict[ini_key + '_is_icecast_json'] == 'True':
-                ice_short_url = GIni.srv_param_dict[url]  # get http://ip:port from full url
-                # ice_short_url = GIni.parse_url_simple_url(url)
-                # call def for reading json file
-                stream_song_name = GNet.is_server_icecast_status_json(ice_short_url, ice_short_url, ini_key)
-
-            if GIni.srv_param_dict[ini_key + '_is_shoutcast_server'] == 'True':
-                srv = GIni.srv_param_dict[url]  # get http://ip:port from full url
-                srv_current = srv + '/currentsong'
-                response = GNet.http_pool.request('GET', srv_current, headers=GNet.ctm_usr_agent, preload_content=False)
-                stream_song_name = response.data.decode('utf-8')
-
-            if GIni.srv_param_dict[ini_key + '_is_no_meta_data_avail'] == 'True':
-                stream_song_name = '_no-name-record_no_split_'
-                # print(' _is_no_meta_data_avail ' + stream_song_name)
+            # hd = request.headers  # try keep conn. open
+            # rand = random.randrange(5, 16, 1)  # sleep random servers (5, 29, 1), helped not much in snap container
 
+            # if GIni.srv_param_dict[ini_key + '_is_unknown_server'] == 'True':
+            #     if not GIni.fail_meta_dict[url] == 'True':  # let record go in one file
+            #         pass
+            #         ret = GRecorder.get_metadata_from_stream(url, ini_key)
+            #         stream_song_name = ret
+            #
+            # if GIni.srv_param_dict[ini_key + '_is_icecast_json'] == 'True':
+            #     ice_short_url = GIni.srv_param_dict[url]  # get http://ip:port from full url
+            #     # ice_short_url = GIni.parse_url_simple_url(url)
+            #     # call def for reading json file
+            #     stream_song_name = GNet.is_server_icecast_status_json(ice_short_url, ice_short_url, ini_key)
+            #
+            # if GIni.srv_param_dict[ini_key + '_is_shoutcast_server'] == 'True':
+            #     srv = GIni.srv_param_dict[url]  # get http://ip:port from full url
+            #     srv_current = srv + '/currentsong'
+            #     response = GNet.http_pool.request('GET', srv_current, headers=GNet.ctm_usr_agent, preload_content=False)
+            #     stream_song_name = response.data.decode('utf-8')
+            #
+            # if GIni.srv_param_dict[ini_key + '_is_no_meta_data_avail'] == 'True':
+            #     stream_song_name = '_no-name-record_no_split_'
+            #     # print(' _is_no_meta_data_avail ' + stream_song_name)
+            #
             if not len(stream_song_name) <= 2:
-                GRecorder.current_song_dict[ini_key] = stream_song_name                        # WRITE SONG NAME
+                #  GRecorder.current_song_dict[ini_key] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
+                stream_song_name = GRecorder.current_song_dict[ini_key]                         # :REVERSE:
 
                 if not update_terminal == stream_song_name:
                     update_terminal = stream_song_name
                     print('\t\t title on ' + ini_key + ':\t' + update_terminal)
             # give recorder command to record, we got a match from search
-            GIni.search_title_keys_list.append(42)
+            # GIni.search_title_keys_list.append(42)     # test value
             try:
                 for _ in GIni.search_title_keys_list:
                     if _ == ini_key:
@@ -578,7 +579,7 @@ class GRecorder:
             except Exception as ex:
                 print(ex)
 
-            for sec in range(rand):
+            for sec in range(GBase.sleeper):
                 sleep(1)
                 if GBase.exit_app:
                     print(f' stop get title {ini_key}')
@@ -588,7 +589,7 @@ class GRecorder:
             #        one file record with name
 
     @staticmethod
-    def search_pattern_start(title, ini_key):
+    def search_pattern_start_record(title, ini_key):
         # search title for strings to see if we should start recording this song/title
         search_list = []  # chop the string
         try:
@@ -626,7 +627,7 @@ class GRecorder:
             pass
 
         sleep(GBase.sleeper + 1)
-        stream_song_name_path = GRecorder.path_to_song_dict[radio_short_key]    # PATH to file
+        stream_song_name_path = GRecorder.path_to_song_dict[radio_short_key]  # PATH to file
 
         try:
             request = GNet.http_pool.request('GET', url, preload_content=False, retries=10)
@@ -700,7 +701,8 @@ class GRecorder:
                                     record_file.flush()
                                     break
 
-                                stream_song_name_path_new = GRecorder.path_to_song_dict[radio_short_key]  # look what's new
+                                stream_song_name_path_new = GRecorder.path_to_song_dict[
+                                    radio_short_key]  # look what's new
 
                                 if not stream_song_name_path_new == '':  # some weired stations send this string
                                     stream_song_name_path = stream_song_name_path_new
@@ -745,7 +747,7 @@ class GRecorder:
                     else:
                         fresh_file_path = directory_save + '//' + clean_name + stream_suffix
                     # write new   FILE PATH   for recorder to make next copy
-                    GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path       # PATH here
+                    GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path  # PATH here
                     # print(f' from head to Tail: {fresh_file_path}')
                     if GBase.exit_app:
                         print(f' exit head {radio_short_key}')
@@ -767,7 +769,7 @@ class GRecorder:
     @staticmethod  # org
     def ghetto_recorder_tail(url, key, path_to_save):
         # shall be as dump as possible
-        #print(f' ghetto_recorder_tail {key} ')
+        # print(f' ghetto_recorder_tail {key} ')
         stream_request_size = io.DEFAULT_BUFFER_SIZE
         old_time = time()
         must_run = False
@@ -787,17 +789,13 @@ class GRecorder:
                         print(f'----> start record: {key}')
                     break
 
-            request = GNet.http_pool.request('GET', url, preload_content=False, retries=10,
-                                             headers={'Icy-MetaData': '1'})
-            icy_metadata = request.headers['icy-metaint']
-            icy_metadata = int(icy_metadata)
-
+            request = GNet.http_pool.request('GET', url, preload_content=False, retries=10)
             while not GBase.exit_app:
                 with open(ghetto_recorder, 'wb') as record_file:
-                    if brake_loop:                       # start_stop_recording stop, can change again to start at run.
+                    if brake_loop:  # start_stop_recording stop, can change again to start at run.
                         break
                     rand = random.randrange(4, 12, 2)  # sleep random
-                    for chunk in request.stream(stream_request_size):                   # create file
+                    for chunk in request.stream(stream_request_size):  # create file
 
                         if GIni.start_stop_recording[key] == 'stop':
                             must_run = False
@@ -828,29 +826,14 @@ class GRecorder:
                                 record_file.truncate()
                                 record_file.seek(0)
 
-                        if time() - old_time > rand:   # random rebuild conn., perhaps it helps in a snap container
+                        ghetto_copy = GRecorder.path_to_song_dict[key]  # SET NEW PATH
+                        if time() - old_time > rand:
                             old_time = time()
-                            # request object to def, get info from stream
-                            request = GNet.http_pool.request('GET', url, preload_content=False, retries=10,
-                                                             headers={'Icy-MetaData': '1'})
-                            request.read(icy_metadata)
-                            chunk_1b = request.read(1)
-                            chunk_1b = ord(chunk_1b)
-                            # \x03 hex value, was div. by 4; hex C (12d) * hex 4 = 30 48d; hex 3 (3d) * 16d = 48d;
-                            read_bytes = chunk_1b * 16
-                            read_bytes = int(read_bytes)
-                            metadata_content = request.read(read_bytes)
-                            metadata_content = metadata_content.decode('utf-8')
-                            title_info = metadata_content.split(";")
-                            title_info = title_info[0].split("=")
-                            title_info = GBase.remove_special_chars(title_info[1])
-                            print(title_info)
-                            break
                         if not chunk:
                             request = GNet.http_pool.request('GET', url, preload_content=False, retries=10)
 
                         if GBase.exit_app:
-                            sleep(2)   # wait for last file name _incomplete_.....
+                            sleep(2)  # wait for last file name _incomplete_.....
                             try:
                                 if os.path.exists(path_to_save + '//__ghetto_trash.mp3'):
                                     os.remove(path_to_save + '//__ghetto_trash.mp3')
@@ -874,12 +857,72 @@ class GRecorder:
                 print(f' exit record {key}')
                 break
 
+    @staticmethod
+    def get_metadata_from_stream_loop(url, key):
+        # urllib3 request.stream with headers={'Icy-MetaData': '1'} , keep open (mdr, swr, br5) test
+        old_time = time()
+
+        while not GBase.exit_app:
+
+            response = GNet.http_pool.request('GET', url,
+                                              headers={'Connection': 'keep-alive'},
+                                              timeout=3000, preload_content=False)
+
+            while not GBase.exit_app:
+                rand_short = random.randrange(4, 12, 2)
+                rand_long = random.randrange(50, 80, 1)  # sleep random
+
+                if time() - old_time > rand_short:
+                    old_time = time()
+                    rh = response.headers
+                    # print(response.headers)
+
+                if time() - old_time > rand_long:  # new connection
+                    old_time = time()
+                    break
+
+                request = GNet.http_pool.request('GET', url,
+                                                 headers={'Icy-MetaData': '1'},
+                                                 timeout=3000, preload_content=False)
+                try:
+                    icy_metadata = request.headers['icy-metaint']
+                    icy_metadata = int(icy_metadata)
+
+                    request.read(icy_metadata)
+                    chunk_1b = request.read(1)
+                    chunk_1b = ord(chunk_1b)
+                    # \x03 hex value, was div. by 4; hex C (12d) * hex 4 = 30 48d; hex 3 (3d) * 16d = 48d;
+                    read_bytes = chunk_1b * 16
+                    read_bytes = int(read_bytes)
+                    metadata_content = request.read(read_bytes)
+                    metadata_content = metadata_content.decode('utf-8')
+                    title_info = metadata_content.split(";")
+                    title_info = title_info[0].split("=")
+                    title_info = GBase.remove_special_chars(title_info[1])
+                    request.release_conn()  # REQUEST not response
+                    # print(f' -> {key} + {title_info}')
+                except Exception as ex:
+                    print(ex)
+                else:
+                    # print(f' -> {key} + {title_info}')
+                    try:
+                        GRecorder.current_song_dict[key] = title_info
+                    except Exception as ex:
+                        print(ex)
+                        pass
+                    else:
+                        for sec in range(rand_short):
+                            if GBase.exit_app:
+                                print(f' out {key} reader')
+                                break
+                            sleep(1)
+
     @staticmethod
     def fire_get_metadata(url, key):
 
         meta_queue = queue.Queue()
         thread1 = threading.Thread(name="thread_meta_daemon",
-                                   target=GRecorder.get_metadata_from_stream_url,   # change def here (url, stream)
+                                   target=GRecorder.get_metadata_from_stream_url,  # change def here (url, stream)
                                    args=(url, key, meta_queue))
         thread1.setDaemon(True)
         thread1.start()
@@ -1183,6 +1226,7 @@ def record(ini_keys):
         GBase.pool.submit(GRecorder.thread_pull_song_name, url, ini_key)
         GBase.pool.submit(GRecorder.ghetto_recorder_head, dir_save, stream_suffix, ini_key)
         GBase.pool.submit(GRecorder.ghetto_recorder_tail, url, ini_key, dir_save)
+        GBase.pool.submit(GRecorder.get_metadata_from_stream_loop, url, ini_key)
 
     while not GBase.exit_app:
         sleep(1)
