@@ -30,12 +30,11 @@ import os
 import queue
 import random
 import signal
-import string
 import sys
 import threading
+from concurrent.futures._base import as_completed
 from time import sleep, strftime, time
 from concurrent.futures import ThreadPoolExecutor, wait
-from concurrent.futures._base import ALL_COMPLETED, Executor, as_completed
 import urllib.request
 import urllib.parse
 from urllib.error import URLError, HTTPError
@@ -68,7 +67,7 @@ import logging
 #  Copy 'run.py' and 'settings.ini' files wherever you want to have your record repository.
 #  Windows let's you double click 'run.py'. Can run from desktop.
 # Magic number 42 is used to connect to all radios found in 'settings.ini'.
-print('0.6.5')
+print('0.6.68')
 
 # https://www.scrapehero.com/how-to-fake-and-rotate-user-agents-using-python-3/
 proxy_support = urllib.request.ProxyHandler({})
@@ -130,7 +129,7 @@ def signal_handler(sig, frame):
 class GBase:
     # class attribute
     exit_app = False
-    sleeper = 2   # for exit of all threads
+    sleeper = 2  # for exit of all threads
     pool = ThreadPoolExecutor(200)
     radio_base_dir = os.path.dirname(os.path.abspath(__file__)) + '//radiostations'  # if not set set_radio_base_dir()
     settings_path = os.path.dirname(os.path.abspath(__file__)) + '//settings.ini'  # if not set in set_settings_path()
@@ -522,46 +521,50 @@ class GRecorder:
         self.url = url
         self.ini_key = ini_key
 
-    @staticmethod  # org pulls only NAME not PATH
+    @staticmethod  # copy keep-alive timeout=3000
     def thread_pull_song_name(url, ini_key):
-        # print(f'thread_pull_song_name: {ini_key}')
+        # print(f' thread_pull_song_name: {ini_key}')
         update_terminal = False
         stream_song_name = GRecorder.current_song_dict[ini_key]
         GIni.fail_meta_dict[url] = 'False'
+        headers = {'Connection': 'keep-alive'}
+        # request = GNet.http_pool.request('GET', url, preload_content=False, retries=10, timeout=3000, headers=headers)
 
         while not GBase.exit_app:
+            # hd = request.headers  # try keep conn. open
+            # rand = random.randrange(5, 16, 1)  # sleep random servers (5, 29, 1), helped not much in snap container
 
-            rand = random.randrange(5, 16, 1)   # sleep random to fool the watch dogs on web servers (5, 29, 1)
-            if GIni.srv_param_dict[ini_key + '_is_unknown_server'] == 'True':
-                if not GIni.fail_meta_dict[url] == 'True':  # let record go in one file
-                    pass
-                    ret = GRecorder.get_metadata_from_stream(url, ini_key)
-                    stream_song_name = ret
-
-            if GIni.srv_param_dict[ini_key + '_is_icecast_json'] == 'True':
-                ice_short_url = GIni.srv_param_dict[url]  # get http://ip:port from full url
-                # ice_short_url = GIni.parse_url_simple_url(url)
-                # call def for reading json file
-                stream_song_name = GNet.is_server_icecast_status_json(ice_short_url, ice_short_url, ini_key)
-
-            if GIni.srv_param_dict[ini_key + '_is_shoutcast_server'] == 'True':
-                srv = GIni.srv_param_dict[url]  # get http://ip:port from full url
-                srv_current = srv + '/currentsong'
-                response = GNet.http_pool.request('GET', srv_current, headers=GNet.ctm_usr_agent, preload_content=False)
-                stream_song_name = response.data.decode('utf-8')
-
-            if GIni.srv_param_dict[ini_key + '_is_no_meta_data_avail'] == 'True':
-                stream_song_name = '_no-name-record_no_split_'
-                # print(' _is_no_meta_data_avail ' + stream_song_name)
-
-            if not len(stream_song_name) <= 2:
-                GRecorder.current_song_dict[ini_key] = stream_song_name                        # WRITE SONG NAME
-
-                if not update_terminal == stream_song_name:
-                    update_terminal = stream_song_name
-                    print('\t\t title on ' + ini_key + ':\t' + update_terminal)
+            # if GIni.srv_param_dict[ini_key + '_is_unknown_server'] == 'True':
+            #     if not GIni.fail_meta_dict[url] == 'True':  # let record go in one file
+            #         pass
+            #         ret = GRecorder.get_metadata_from_stream(url, ini_key)
+            #         stream_song_name = ret
+            #
+            # if GIni.srv_param_dict[ini_key + '_is_icecast_json'] == 'True':
+            #     ice_short_url = GIni.srv_param_dict[url]  # get http://ip:port from full url
+            #     # ice_short_url = GIni.parse_url_simple_url(url)
+            #     # call def for reading json file
+            #     stream_song_name = GNet.is_server_icecast_status_json(ice_short_url, ice_short_url, ini_key)
+            #
+            # if GIni.srv_param_dict[ini_key + '_is_shoutcast_server'] == 'True':
+            #     srv = GIni.srv_param_dict[url]  # get http://ip:port from full url
+            #     srv_current = srv + '/currentsong'
+            #     response = GNet.http_pool.request('GET', srv_current, headers=GNet.ctm_usr_agent, preload_content=False)
+            #     stream_song_name = response.data.decode('utf-8')
+            #
+            # if GIni.srv_param_dict[ini_key + '_is_no_meta_data_avail'] == 'True':
+            #     stream_song_name = '_no-name-record_no_split_'
+            #     # print(' _is_no_meta_data_avail ' + stream_song_name)
+            #
+            # if not len(stream_song_name) <= 2:
+            #     #  GRecorder.current_song_dict[ini_key] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
+            #     stream_song_name = GRecorder.current_song_dict[ini_key]
+            #
+            #     if not update_terminal == stream_song_name:
+            #         update_terminal = stream_song_name
+            #         print('\t\t title on ' + ini_key + ':\t' + update_terminal)
             # give recorder command to record, we got a match from search
-            GIni.search_title_keys_list.append(42)
+            # # GIni.search_title_keys_list.append(42)     # test value
             try:
                 for _ in GIni.search_title_keys_list:
                     if _ == ini_key:
@@ -575,7 +578,7 @@ class GRecorder:
             except Exception as ex:
                 print(ex)
 
-            for sec in range(rand):
+            for sec in range(GBase.sleeper):
                 sleep(1)
                 if GBase.exit_app:
                     print(f' stop get title {ini_key}')
@@ -623,7 +626,7 @@ class GRecorder:
             pass
 
         sleep(GBase.sleeper + 1)
-        stream_song_name_path = GRecorder.path_to_song_dict[radio_short_key]    # PATH to file
+        stream_song_name_path = GRecorder.path_to_song_dict[radio_short_key]  # PATH to file
 
         try:
             request = GNet.http_pool.request('GET', url, preload_content=False, retries=10)
@@ -697,7 +700,8 @@ class GRecorder:
                                     record_file.flush()
                                     break
 
-                                stream_song_name_path_new = GRecorder.path_to_song_dict[radio_short_key]  # look what's new
+                                stream_song_name_path_new = GRecorder.path_to_song_dict[
+                                    radio_short_key]  # look what's new
 
                                 if not stream_song_name_path_new == '':  # some weired stations send this string
                                     stream_song_name_path = stream_song_name_path_new
@@ -731,8 +735,8 @@ class GRecorder:
 
                     fresh_song = stream_song_name  # check if it can be removed, we look in a dict now
                     clean_name = GBase.remove_special_chars(stream_song_name)
-                    if clean_name == '_no-name-record_no_split_':  # init name of title on startup
-                        clean_name = clean_name + GBase.this_time()
+                    if clean_name == 'untitled_full_record_':  # init name of title on startup
+                        clean_name = clean_name + radio_short_key + '_date_' + GBase.this_time()
                         clean_name = GBase.remove_special_chars(clean_name)
 
                     if first_record:
@@ -742,7 +746,7 @@ class GRecorder:
                     else:
                         fresh_file_path = directory_save + '//' + clean_name + stream_suffix
                     # write new   FILE PATH   for recorder to make next copy
-                    GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path       # PATH here
+                    GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path  # PATH here
                     # print(f' from head to Tail: {fresh_file_path}')
                     if GBase.exit_app:
                         print(f' exit head {radio_short_key}')
@@ -764,7 +768,7 @@ class GRecorder:
     @staticmethod  # org
     def ghetto_recorder_tail(url, key, path_to_save):
         # shall be as dump as possible
-        # print(f'ghetto_recorder_tail {key} ')
+        # print(f' ghetto_recorder_tail {key} ')
         stream_request_size = io.DEFAULT_BUFFER_SIZE
         old_time = time()
         must_run = False
@@ -787,10 +791,10 @@ class GRecorder:
             request = GNet.http_pool.request('GET', url, preload_content=False, retries=10)
             while not GBase.exit_app:
                 with open(ghetto_recorder, 'wb') as record_file:
-                    if brake_loop:                       # start_stop_recording stop, can change again to start at run.
+                    if brake_loop:  # start_stop_recording stop, can change again to start at run.
                         break
-
-                    for chunk in request.stream(stream_request_size):                   # create file
+                    rand = random.randrange(4, 12, 2)  # sleep random
+                    for chunk in request.stream(stream_request_size):  # create file
 
                         if GIni.start_stop_recording[key] == 'stop':
                             must_run = False
@@ -822,14 +826,13 @@ class GRecorder:
                                 record_file.seek(0)
 
                         ghetto_copy = GRecorder.path_to_song_dict[key]  # SET NEW PATH
-                        if time() - old_time > 20:
+                        if time() - old_time > rand:
                             old_time = time()
-                            # Do something (trigger header change?)
                         if not chunk:
                             request = GNet.http_pool.request('GET', url, preload_content=False, retries=10)
 
                         if GBase.exit_app:
-                            sleep(2)   # wait for last file name _incomplete_.....
+                            sleep(2)  # wait for last file name _incomplete_.....
                             try:
                                 if os.path.exists(path_to_save + '//__ghetto_trash.mp3'):
                                     os.remove(path_to_save + '//__ghetto_trash.mp3')
@@ -853,12 +856,66 @@ class GRecorder:
                 print(f' exit record {key}')
                 break
 
+    @staticmethod
+    def get_metadata_from_stream_loop(url, key):
+        # urllib3 request.stream with headers={'Icy-MetaData': '1'} , keep open (mdr, swr, br5) test
+        old_time = time()
+
+        while not GBase.exit_app:
+
+            response = GNet.http_pool.request('GET', url,
+                                              headers={'Connection': 'keep-alive'},
+                                              timeout=3000, preload_content=False)
+
+            while not GBase.exit_app:
+                rand_short = random.randrange(4, 12, 2)
+                rand_long = random.randrange(50, 80, 1)  # sleep random
+
+                if time() - old_time > rand_short:
+                    old_time = time()
+                    rh = response.headers
+                    # print(response.headers)
+
+                if time() - old_time > rand_long:  # new connection
+                    old_time = time()
+                    break
+
+                request = GNet.http_pool.request('GET', url,
+                                                 headers={'Icy-MetaData': '1'},
+                                                 timeout=3000, preload_content=False)
+                try:
+                    icy_metadata = request.headers['icy-metaint']
+                    icy_metadata = int(icy_metadata)
+
+                    request.read(icy_metadata)
+                    chunk_1b = request.read(1)
+                    chunk_1b = ord(chunk_1b)
+                    # \x03 hex value, was div. by 4; hex C (12d) * hex 4 = 30 48d; hex 3 (3d) * 16d = 48d;
+                    read_bytes = chunk_1b * 16
+                    read_bytes = int(read_bytes)
+                    metadata_content = request.read(read_bytes)
+                    metadata_content = metadata_content.decode('utf-8')
+                    title_info = metadata_content.split(";")
+                    title_info = title_info[0].split("=")
+                    title_info = GBase.remove_special_chars(title_info[1])
+                    request.release_conn()
+                    # print(f' -> {key} + {title_info}')
+                except Exception as ex:
+                    print(ex)
+                else:
+                    print(f' -> {key} + {title_info}')
+                    GRecorder.current_song_dict[key] = title_info
+                    for sec in range(rand_short):
+                        if GBase.exit_app:
+                            break
+                        sleep(1)
+
     @staticmethod
     def fire_get_metadata(url, key):
 
         meta_queue = queue.Queue()
         thread1 = threading.Thread(name="thread_meta_daemon",
-                                   target=GRecorder.get_metadata_from_stream_url,   # change def here (url, stream)
+                                   target=GRecorder.get_metadata_from_stream_url,  # change def here (url, stream)
                                    args=(url, key, meta_queue))
         thread1.setDaemon(True)
         thread1.start()
@@ -1162,6 +1219,7 @@ def record(ini_keys):
         GBase.pool.submit(GRecorder.thread_pull_song_name, url, ini_key)
         GBase.pool.submit(GRecorder.ghetto_recorder_head, dir_save, stream_suffix, ini_key)
         GBase.pool.submit(GRecorder.ghetto_recorder_tail, url, ini_key, dir_save)
+        GBase.pool.submit(GRecorder.get_metadata_from_stream_loop, url, ini_key)
 
     while not GBase.exit_app:
         sleep(1)
